#!/usr/bin/env python
#
# -------------------------------------------------------------------------------------
#
# Copyright (c) 2018 KGH
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#
# This file is a basic Python plug-in for GIMP.
#
# It can be executed by selecting the menu option: 'Filters>>Custom Font Tools>>Custom Font Distort Pack 1 v0.01...'
#
# Change Log
# Version 0.01 Provides a means of batch calling a number of distort filters in Gimp 2.10 for the following plug-ins
#              ripple plug-in, waves plug-in, shift plug-in and wind plug-in
#              Added color-to-alpha to results of wind plug-in 

from gimpfu import *

def do_stuff(image, layer, constrain_selection, use_ripple, ripple_period, ripple_amplitude, ripple_orientation, use_waves, wave_amplitude, wave_phase, wave_length, 
            use_shift, shift_amount, shift_orientation, use_wind, wind_direction, wind_strength, wind_threshold, wind_style, wind_edge):

    # Store the GIMP's settings so they can be restored
    gimp.context_push()
    # Set start undo point
    pdb.gimp_image_undo_group_start(image)
	
    #use_clipboard_as_brush, do_layer_info, do_layer_names, do_path_names, do_sort = 0,0,0,0,0
    outline_colour = pdb.gimp_context_get_foreground()
	
    # Define some colours for proc chisel
    black = gimpcolor.RGB(0, 0, 0)
    white = gimpcolor.RGB(255, 255, 255)
    midgrey = gimpcolor.RGB(125, 125, 125)
	
    # Temp variables
    use_texture = False

    loop_ptr = 1
    num_layers, layer_ids = pdb.gimp_image_get_layers(image)
    loop = num_layers - loop_ptr
    layer_names_list = image.layers
	
    first_char_layer_name = "none"
	
    if (use_ripple or use_waves or use_shift or use_wind):
        # turn off visibility of layers
        visi_loop, first_char_layer  = 0, -1
        while visi_loop < num_layers:
	        # get info about layer we are working on
            layer_name = pdb.gimp_item_get_name(layer_names_list[visi_loop])
            layer_name = layer_name.decode('utf-8')
            if len(layer_name) == 1 or len(layer_name) == 5 and ".png" in layer_name:
                if first_char_layer < 0: 
                    first_char_layer = visi_loop
                    first_char_layer_name = layer_name
            pdb.gimp_item_set_visible(layer_names_list[visi_loop],False)
            visi_loop += 1
	
    while loop >= 0:
	    # get info about layer we are working on
        layer_id = layer_ids[loop]
        layer_name = layer_names_list[loop]
	
        # cater for unicode characters above normal range
        display_layer_name =  pdb.gimp_item_get_name(layer_names_list[loop])
        display_layer_name = display_layer_name.decode('utf-8')	
	
        # check we have the layer id of the currently active layer
        pdb.gimp_image_set_active_layer(image, layer_name)
        layer_id = pdb.gimp_image_get_active_layer(image)

        # check to see if the current layer is a custom font character layer
        test_for_custom_layer = fn_test_for_custom_layer(image, layer_id, display_layer_name)
		
        # if the outline width has been set to zero but no chisel or shine has been set don't do it
        #if use_chisel == False and use_shine == False: test_for_custom_layer = False
		
        if test_for_custom_layer:

            # ensure we have the active layer and make it visible
            active_layer_id = pdb.gimp_image_get_active_layer(image)
            pdb.gimp_item_set_visible(active_layer_id, True)

            if use_ripple:
                active_layer = image.active_layer
                ripple_edges = 0
                ripple_waveform = 1
                antialiasing = True
                ripple_tile = False			

                active_layer_id = proc_do_ripple(image, active_layer, constrain_selection, ripple_period, ripple_amplitude, ripple_orientation, ripple_edges, ripple_waveform, antialiasing, ripple_tile)
				
				
            if use_waves:
                active_layer = image.active_layer
                wave_type = True
                wave_reflective = False

                active_layer_id = proc_do_waves(image, active_layer, constrain_selection, wave_amplitude, wave_phase, wave_length, wave_type, wave_reflective)
					
				
            if use_shift:
                active_layer = image.active_layer
                if shift_orientation == 0:
                    orientation = 1
                else:
                    orientation = 0
                active_layer_id = proc_do_shift(image, active_layer, shift_amount, orientation)	
				
				
            if use_wind:
                active_layer = image.active_layer
                active_layer_id = proc_do_wind(image, active_layer, wind_threshold, wind_direction, wind_strength, wind_style, wind_edge)
                active_layer = image.active_layer	
                num_layers, layer_ids = pdb.gimp_image_get_layers(image)
					
            if (num_layers - loop_ptr) > 0: pdb.gimp_item_set_visible(active_layer_id, False)
			
            num_layers, layer_ids = pdb.gimp_image_get_layers(image)
			
        loop_ptr += 1
        loop = num_layers - loop_ptr
        
    if use_texture: 
        image.remove_layer(bumpmap_layer)
        active_layer = image.active_layer 

    if first_char_layer_name <> "none":
        first_char_layer_id = pdb.gimp_image_get_layer_by_name(image, first_char_layer_name)
        pdb.gimp_item_set_visible(first_char_layer_id, True)
        pdb.gimp_image_set_active_layer(image,first_char_layer_id)
	
    # Set end undo point
    pdb.gimp_image_undo_group_end(image)

    # Restore the GIMP's settings
    gimp.context_pop()
		
#### End of do_stuff

######################################################################################################################################

def proc_do_wind(image, active_layer, wind_threshold, wind_direction, wind_strength, wind_style, wind_edge):
    #if constrain: pdb.gimp_image_select_item(image, 2, active_layer)
    pdb.plug_in_wind(image, active_layer, wind_threshold, wind_direction, wind_strength, wind_style, wind_edge)
    active_layer = image.active_layer 
    # Select outline
    pdb.gimp_image_select_item(image, 2, active_layer)
    # Invert selection
    pdb.gimp_selection_invert(image)
    # Change white pixels that lie outside the character outline to transparent 
    pdb.plug_in_colortoalpha(image, active_layer, gimpcolor.RGB(255, 255, 255))
    active_layer = image.active_layer
    # Remove selection
    pdb.gimp_selection_none(image)
    # Repeat this white to transparent two more times
    pdb.gimp_image_select_item(image, 2, active_layer)
    pdb.gimp_selection_invert(image)
    pdb.plug_in_colortoalpha(image, active_layer, gimpcolor.RGB(255, 255, 255))
    active_layer = image.active_layer 
    pdb.gimp_selection_none(image)
	
    pdb.gimp_image_select_item(image, 2, active_layer)
    pdb.gimp_selection_invert(image)
    pdb.plug_in_colortoalpha(image, active_layer, gimpcolor.RGB(255, 255, 255))
    active_layer = image.active_layer 
    pdb.gimp_selection_none(image)
	
    #if constrain: pdb.gimp_selection_none(image)
    return active_layer	


def proc_do_shift(image, active_layer, shift_amount, shift_orientation):
    #if constrain: pdb.gimp_image_select_item(image, 2, active_layer)
    pdb.plug_in_shift(image, active_layer, shift_amount, shift_orientation)
    #if constrain: pdb.gimp_selection_none(image)
    return active_layer	

def proc_do_mosaic(image, active_layer, tile_size, tile_height, tile_spacing, tile_neatness, tile_allow_split, azimuth, tile_colour_variation, antialiasing, tile_colour_averaging, tile_type, tile_surface, tile_grout_colour):
    
    # select the contents of the layer
    pdb.gimp_image_select_item(image, 2, active_layer)
    pdb.plug_in_mosaic(image, active_layer, tile_size, tile_height, tile_spacing, tile_neatness, tile_allow_split, azimuth, tile_colour_variation, antialiasing, tile_colour_averaging, tile_type, tile_surface, tile_grout_colour)
    pdb.gimp_selection_none(image)
    return active_layer	
	
def proc_do_ripple(image, active_layer, constrain, ripple_period, ripple_amplitude, ripple_orientation, ripple_edges, ripple_waveform, antialiasing, ripple_tile):
    if constrain: pdb.gimp_image_select_item(image, 2, active_layer)
    pdb.plug_in_ripple(image, active_layer, ripple_period, ripple_amplitude, ripple_orientation, ripple_edges, ripple_waveform, antialiasing, ripple_tile)
    if constrain: pdb.gimp_selection_none(image)
    return active_layer	
	
def proc_do_waves(image, active_layer, constrain, wave_amplitude, wave_phase, wave_length, wave_type, wave_reflective):
    if constrain: pdb.gimp_image_select_item(image, 2, active_layer)
    pdb.plug_in_waves(image, active_layer, wave_amplitude, wave_phase, wave_length, wave_type, wave_reflective)
    if constrain: pdb.gimp_selection_none(image)
    return active_layer	

######################################################################################################################################

def proc_trim_away(img, layer, amount):    
    # select the contents of the layer
    pdb.gimp_image_select_item(img, 2, layer)
    # shrink the selection
    pdb.gimp_selection_shrink(img, amount)
    # invert selection
    pdb.gimp_selection_invert(img)
    # trim away pixels
    successful = pdb.gimp_edit_cut(layer)
    # deselect
    pdb.gimp_selection_none(img)
    
######################################################################################################################################

def fn_test_for_custom_layer(img,l_id,l_name):
    test = False
    if len(l_name) == 1:
        test = True
    elif len(l_name) == 5:
        if ".png" in l_name:
            test = True
    # select the contents of the layer
    pdb.gimp_image_select_item(img,2,l_id)
    # Test to see if the selection actually contains something
    sel_is_empty = pdb.gimp_selection_is_empty(img)
    # If there are no pixels to work with flag the layer as not viable to work on
    if sel_is_empty: test = False
    # Deselect
    pdb.gimp_selection_none(img)
    return test
	
######################################################################################################################################
	
def fn_set_bevel_mode(bevel_mode_ptr):
    # Use New Modes when creating a bevel
    bevel_mode = 23
    if bevel_mode_ptr == 0:
        bevel_mode = 23 #Overlay Mode
    elif bevel_mode_ptr == 1:
        bevel_mode = 30 #Multiply Mode
    elif bevel_mode_ptr == 2:
        bevel_mode = 35 #Darken Only Mode
    elif bevel_mode_ptr == 3:
        bevel_mode = 40 #HSV Value Mode
    elif bevel_mode_ptr == 4:
        bevel_mode = 41 #Divide Mode
    elif bevel_mode_ptr == 5:
        bevel_mode = 44 # Hardlight Mode
    elif bevel_mode_ptr == 6:
        bevel_mode = 45 # Softlight Mode
    if bevel_mode_ptr == 7:
        bevel_mode = 46 #Grain Extract Mode
    elif bevel_mode_ptr == 8:
        bevel_mode = 47 #Grain Merge Mode
    elif bevel_mode_ptr == 9:
        bevel_mode = 48 #Vivid Mode
    elif bevel_mode_ptr == 10:
        bevel_mode = 50 #Linear Light Mode
    elif bevel_mode_ptr == 11:
        bevel_mode = 51 #Hard Mix Mode
    elif bevel_mode_ptr == 12:
        bevel_mode = 56 #Luminance Mode
    return bevel_mode


######################################################################################################################################

register(
    "python_fu_Custom_Font_Distortion_Pack_1_v0-01",
    "Adds distortion plug-ins to custom fonts",
    "Provides options for adding batch distortion plug-ins to custom fonts",
    "Skinnyhouse",
    "Open source (BSD 3-clause license)",
    "2018",
    "<Image>/Filters/Custom Font Tools/Custom Font Distortion Pack 1 v0.01...",
    "*",
    [
    (PF_TOGGLE, "pf_constrain", "Constrain effect to letter shape: ", 0),
    (PF_TOGGLE, "pf_add_ripple", "Add ripple effect: ", 0),
    (PF_SPINNER, "pf_ripple_period", "Ripple period: ", 200.0,(1.0,1000.0,1.0)),
    (PF_SPINNER, "pf_ripple_amplitude", "Ripple amplitude: ", 25.0,(1.0,1000.0,1.0)),
    (PF_OPTION, "pf_ripple_orientation", "Ripple orientation: ", 0, ("horizontal", "vertical")),
    (PF_TOGGLE, "pf_add_wave", "Add wave effect: ", 0),
    (PF_SPINNER, "pf_wave_amplitude", "Wave amplitude: ", 25.0,(0.0,101.0,1.0)),
    (PF_SPINNER, "pf_wave_phase", "Wave Phase: ", 0.0,(-360.0,360.0,1.0)),
    (PF_SPINNER, "pf_wave_length", "Wave length: ", 25.0,(0.1,50.0,0.1)),
    (PF_TOGGLE, "pf_add_shift", "Add Shift effect: ", 0),
    (PF_SPINNER, "pf_shift_amount", "Shift amount: ", 15,(0,200,1)),
    (PF_OPTION, "pf_shift_orientation", "Shift orientation: ", 0, ("horizontal", "vertical")),
    (PF_TOGGLE, "pf_add_wind", "Add Wind effect: ", 0),
    (PF_OPTION, "pf_wind_direction", "Wind direction: ", 0, ("left", "right", "top", "bottom")),
    (PF_SPINNER, "pf_wind_strength", "Wind strength: ", 10,(1,100,1)),
    (PF_SPINNER, "pf_wind_threshold", "Wind threshold: ", 10,(0,50,1)),
    (PF_OPTION, "pf_wind_style", "Wind style: ", 0, ("wind", "blast")),
    (PF_OPTION, "pf_wind_edge", "Edge affected: ", 1, ("both edges", "leading edge", "trailing edge")),
    ],
    [],
    do_stuff)

main()

	

