#!/usr/bin/env python
#
# -------------------------------------------------------------------------------------
#
# Copyright (c) 2018 KGH
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#
# This file is a basic Python plug-in for GIMP.
#
# It can be executed by selecting the menu option: 'Filters>>Custom Font Tools>>Custom Font Distort Pack 2 v0.01...'
#
# Change Log
# Version 0.01 Provides batch calls to a number of distort filters in Gimp 2.10 for the following plug-ins
#              mosaic plug-in, video degradation plug-in and whirl and pinch plug-in

from gimpfu import *

def do_stuff(image, layer, constrain_selection, use_mosaic, effect_azimuth, mosaic_tile_type, mosaic_tile_size, mosaic_tile_spacing, mosaic_colour_variation, mosaic_tile_surface, use_video_patterns, vid_pattern_type, vid_rotate, 
            use_whirl, whirl_angle, pinch_amount, whirl_radius):

    # Store the GIMP's settings so they can be restored
    gimp.context_push()
    # Set start undo point
    pdb.gimp_image_undo_group_start(image)
	
    #use_clipboard_as_brush, do_layer_info, do_layer_names, do_path_names, do_sort = 0,0,0,0,0
    outline_colour = pdb.gimp_context_get_foreground()
	
    # Define some colours for proc chisel
    black = gimpcolor.RGB(0, 0, 0)
    white = gimpcolor.RGB(255, 255, 255)
    midgrey = gimpcolor.RGB(125, 125, 125)
	
    # Temp variables
    use_texture = False

    loop_ptr = 1
    num_layers, layer_ids = pdb.gimp_image_get_layers(image)
    loop = num_layers - loop_ptr
    layer_names_list = image.layers
	
    visi_loop, other_layers, first_letter_layer, has_a_space = 0, 0, -1, False
	
    first_char_layer_name = "none"
	
    if (use_mosaic or use_video_patterns or use_whirl):
        # turn off visibility of layers
        visi_loop, first_char_layer  = 0, -1
        while visi_loop < num_layers:
	        # get info about layer we are working on
            layer_name = pdb.gimp_item_get_name(layer_names_list[visi_loop])
            layer_name = layer_name.decode('utf-8')
            if len(layer_name) == 1 or len(layer_name) == 5 and ".png" in layer_name:
                if first_char_layer < 0: 
                    first_char_layer = visi_loop
                    first_char_layer_name = layer_name
            pdb.gimp_item_set_visible(layer_names_list[visi_loop],False)
            visi_loop += 1
	
    while loop >= 0:
	    # get info about layer we are working on
        layer_id = layer_ids[loop]
        layer_name = layer_names_list[loop]
	
        # cater for unicode characters above normal range
        display_layer_name =  pdb.gimp_item_get_name(layer_names_list[loop])
        display_layer_name = display_layer_name.decode('utf-8')	
	
        # check we have the layer id of the currently active layer
        pdb.gimp_image_set_active_layer(image, layer_name)
        layer_id = pdb.gimp_image_get_active_layer(image)

        # check to see if the current layer is a custom font character layer
        test_for_custom_layer = fn_test_for_custom_layer(image, layer_id, display_layer_name)
		
        # if the outline width has been set to zero but no chisel or shine has been set don't do it
        #if use_chisel == False and use_shine == False: test_for_custom_layer = False
		
        if test_for_custom_layer:

            # ensure we have the active layer and make it visible
            active_layer_id = pdb.gimp_image_get_active_layer(image)
            pdb.gimp_item_set_visible(active_layer_id, True)
			
            if use_whirl:
                active_layer = image.active_layer
				
                active_layer_id = proc_do_whirl(image, active_layer, constrain_selection, whirl_angle, pinch_amount, whirl_radius)

            if use_mosaic:
                active_layer = image.active_layer
                tile_size = mosaic_tile_size
                tile_height = 4.0
                tile_spacing = mosaic_tile_spacing
                tile_neatness = 0.650
                tile_allow_split = True
                tile_colour_variation = mosaic_colour_variation
                antialiasing = True
                tile_colour_averaging = True
                tile_type = mosaic_tile_type
                tile_surface = mosaic_tile_surface
                tile_grout_colour = True
                
                active_layer_id = proc_do_mosaic(image, active_layer, tile_size, tile_height, tile_spacing, tile_neatness, tile_allow_split, effect_azimuth, tile_colour_variation, antialiasing, tile_colour_averaging, tile_type, tile_surface, tile_grout_colour)
	
            if use_video_patterns:
                active_layer = image.active_layer
                video_additive = True
				
                active_layer_id = proc_do_video_degradation(image, active_layer, vid_pattern_type, video_additive, vid_rotate)
				
	
					
            if (num_layers - loop_ptr) > 0: pdb.gimp_item_set_visible(active_layer_id, False)
			
            num_layers, layer_ids = pdb.gimp_image_get_layers(image)
			
        loop_ptr += 1
        loop = num_layers - loop_ptr
        
    if use_texture: 
        image.remove_layer(bumpmap_layer)
        active_layer = image.active_layer 

    if first_char_layer_name <> "none":
        first_char_layer_id = pdb.gimp_image_get_layer_by_name(image, first_char_layer_name)
        pdb.gimp_item_set_visible(first_char_layer_id, True)
        pdb.gimp_image_set_active_layer(image,first_char_layer_id)
	
    # Set end undo point
    pdb.gimp_image_undo_group_end(image)

    # Restore the GIMP's settings
    gimp.context_pop()
		
#### End of do_stuff

######################################################################################################################################

def proc_do_whirl(image, active_layer, constrain, whirl_angle, pinch_amount, whirl_radius):
    if constrain: pdb.gimp_image_select_item(image, 2, active_layer)
    pdb.plug_in_whirl_pinch(image, active_layer, whirl_angle, pinch_amount, whirl_radius)
    if constrain: pdb.gimp_selection_none(image)
    return active_layer	
	
def proc_do_video_degradation(image, active_layer, vid_pattern_type, video_additive, vid_rotate):
    #if constrain: pdb.gimp_image_select_item(image, 2, active_layer)
    pdb.plug_in_video(image, active_layer,  vid_pattern_type, video_additive, vid_rotate)
    #if constrain: pdb.gimp_selection_none(image)
    return active_layer	

def proc_do_mosaic(image, active_layer, tile_size, tile_height, tile_spacing, tile_neatness, tile_allow_split, azimuth, tile_colour_variation, antialiasing, tile_colour_averaging, tile_type, tile_surface, tile_grout_colour):
    
    # select the contents of the layer
    pdb.gimp_image_select_item(image, 2, active_layer)
    pdb.plug_in_mosaic(image, active_layer, tile_size, tile_height, tile_spacing, tile_neatness, tile_allow_split, azimuth, tile_colour_variation, antialiasing, tile_colour_averaging, tile_type, tile_surface, tile_grout_colour)
    pdb.gimp_selection_none(image)
    return active_layer	
	
######################################################################################################################################
	
def proc_trim_away(img, layer, amount):    
    # select the contents of the layer
    pdb.gimp_image_select_item(img, 2, layer)
    # shrink the selection
    pdb.gimp_selection_shrink(img, amount)
    # invert selection
    pdb.gimp_selection_invert(img)
    # trim away pixels
    successful = pdb.gimp_edit_cut(layer)
    # deselect
    pdb.gimp_selection_none(img)
    
######################################################################################################################################

def fn_test_for_custom_layer(img,l_id,l_name):
    test = False
    if len(l_name) == 1:
        test = True
    elif len(l_name) == 5:
        if ".png" in l_name:
            test = True
    # select the contents of the layer
    pdb.gimp_image_select_item(img,2,l_id)
    # Test to see if the selection actually contains something
    sel_is_empty = pdb.gimp_selection_is_empty(img)
    # If there are no pixels to work with flag the layer as not viable to work on
    if sel_is_empty: test = False
    # Deselect
    pdb.gimp_selection_none(img)
    return test
	
######################################################################################################################################

register(
    "python_fu_Custom_Font_Distortion_Pack_2_v0-01",
    "Adds distortion plug-ins to custom fonts",
    "Provides options for adding batch distortion plug-ins to custom fonts",
    "Skinnyhouse",
    "Open source (BSD 3-clause license)",
    "2018",
    "<Image>/Filters/Custom Font Tools/Custom Font Distortion Pack 2 v0.01...",
    "*",
    [
    (PF_TOGGLE, "pf_constrain", "Constrain effect to letter shape: ", 0),
    (PF_TOGGLE, "pf_add_mosaic", "Add mosaic effect: ", 0),
    (PF_SPINNER, "pf_shine_azimuth", "Mosaic azimuth: ", 315,(0,360,1)),
    (PF_OPTION, "pf_tile_type", "Tile shape: ", 2, ("squares", "hexagons","octagons","triangles")),
    (PF_SPINNER, "pf_tile_size", "Tile size: ", 15.0,(1.00,1000.0,1.0)),
    (PF_SPINNER, "pf_tile_spacing", "Tile spacing (BG colour): ", 1.0,(0.5,10.0,0.1)),
    (PF_SPINNER, "pf_tile_colour_variation", "Tile colour variation: ", 0.2,(0.0,1.0,0.1)),
    (PF_OPTION, "pf_tile_surface", "Tile surface: ", 0, ("smooth", "rough (FG colour)",)),
    (PF_TOGGLE, "pf_add_degradation", "Add video degradation effect: ", 0),
    (PF_OPTION, "pf_vid_pattern_type", "Pattern to use: ", 2, ("staggered","large staggered","striped","wide striped","long staggered","3x3","large 3x3","hex","dots")),
    (PF_TOGGLE, "pf_vid_rotate", "Rotate the pattern 90 degrees: ", 0),	
    (PF_TOGGLE, "pf_add_whirl", "Add whirl & pinch effect: ", 0),
    (PF_SPINNER, "pf_whirl_angle", "Whirl angle: ", 90.0,(-720.0,720.0,1.0)),
    (PF_SPINNER, "pf_whirl_pinch", "Pinch amount: ", 0.000,(-1.000,1.000,0.001)),
    (PF_SPINNER, "pf_whirl_radius", "Whirl radius factor: ", 1.00,(0.00,2.00,0.01)),
    #(PF_SPINNER, "pf_emboss_blur", "Emboss blur: ", 5,(0,100,1)),
    #(PF_TOGGLE, "pf_add_bump", "Add Bumpmapped Texture: ", 0),
    #(PF_SPINNER, "pf_bump_depth", "Bumpmap depth (percentage): ", 30,(0,100,1)),
    #(PF_SPINNER, "pf_bump_waterlevel", "Bumpmap waterlevel (percentage): ", 0,(0,100,1)),
    #(PF_SPINNER, "pf_bump_ambient", "Bumpmap ambient light (percentage): ", 0,(0,100,1)),
    #(PF_TOGGLE, "pf_add_shine", "Add Shine effect: ", 0),
    #(PF_SPINNER, "pf_shine_trim", "Trim shine in pixels: ", 0,(0,10,1)),
    #(PF_TOGGLE, "pf_shine_colour", "Use (BG) colour for Shine: ", 0),
    #(PF_TOGGLE, "pf_add_bevel", "Add a bevel: ", 0),
    #(PF_TOGGLE, "pf_bevel_out_or_bevel_in_mode", "Bevel out (Yes) or bevel in (No): ", 1),
    #(PF_SPINNER, "pf_bevel_width", "Bevel width: ", 30,(0,100,1)),
    #(PF_SPINNER, "pf_bevel_depth", "Bevel depth: ", 40,(0,100,1)),
    #(PF_SPINNER, "pf_bevel_waterlevel", "Bevel waterlevel (percentage): ", 0,(0,100,1)),
    #(PF_SPINNER, "pf_bevel_ambient", "Bevel ambient light (percentage): ", 0,(0,100,1)),
    #(PF_SPINNER, "pf_blur_radius", "Bevel blur radius: ", 10,(0.0,100.0,0.1)),
    #(PF_SPINNER, "pf_blur_delta", "Bevel blur delta amount: ", 8,(0,100,1)),
    #(PF_SPINNER, "pf_effect_elevation", "Effect elevation: ", 45.0,(0.5,90,0.1)),
    #(PF_SPINNER, "pf_shine_azimuth", "Effect azimuth: ", 315,(0,360,1)),
    #(PF_OPTION, "pf_option", "Effect colour mode: ", 8, ("Overlay Mode", "Multiply Mode", "Darken Only Mode", "HSV Value Mode", "Divide Mode", "Dodge Mode", "Hardlight Mode", "Softlight Mode", "Grain Extract Mode", "Grain Merge Mode", "Vivid Light Mode", "Hard Mix Mode", "Luminance Mode")),
    ],
    [],
    do_stuff)

main()

	

