#!/usr/bin/env python
#
# -------------------------------------------------------------------------------------
#
# Copyright (c) 2018 KGH
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#
# This file is a basic Python plug-in for GIMP.
#
# It can be executed by selecting the menu option: 'Filters>>Custom Font Tools>>Effects Only v0.01...'
#
# Change Log
# Version 0.01 Provision added to call Shine Filter v 0.01 or v 0.01b by Graechan and Chisel or Carve Filter version 1.3 by Rob Antonishen
# Version 0.02 Outline width set to zero will allow the filter to add shine and/or chisel to existing custom font layers without adding outlines when outline width set to zero
#              Distort will also work on all these outlines apart from the brushed outline when outline width set to zero and Distort selection spread value > 0
# Version 0.03 Trimmed pixels from letter faces to remove slight halo effect
# Version 0.04 Added gaussian blur after chisel effect to smooth artifacts using pdb.plug_in_sel_blur with options for blur radius and blur delta
# Version 0.05 Removed dependency on shine and chisel plug-ins and replaced calls with embedded code
#              Added option to colour the shiny gloss

# Current Version of Custom Font Outlines 0.05
# Version 0.01 Effects Only
#              Separated from Outlines to leave only Shine and Chisel or Carve effects to reduce dialog height.
# Version 0.02 Effects Only
#              Added Emboss and Bumpmap Options
# Version 0.03 Effects Only
#              Added separate options for effects
# Version 0.04 Effects Only
#              Added trim option with shine effect for user to remove pixels from the character borders

from gimpfu import *

def do_stuff(image, layer, use_emboss, emboss_width, emboss_blur, use_texture, bump_depth, bump_waterlevel, bump_ambient, use_shine, trim_value, gloss_colour, use_bevel, bevel_mode_ptr, bevel_width, bevel_depth, bevel_waterlevel, bevel_ambient, blur_radius, blur_delta, effect_elevation, effect_azimuth, effect_mode_ptr):

    # Store the GIMP's settings so they can be restored
    gimp.context_push()
    # Set start undo point
    pdb.gimp_image_undo_group_start(image)
	
    #use_clipboard_as_brush, do_layer_info, do_layer_names, do_path_names, do_sort = 0,0,0,0,0
    outline_colour = pdb.gimp_context_get_foreground()
	
    # Define some colours for proc chisel
    black = gimpcolor.RGB(0, 0, 0)
    white = gimpcolor.RGB(255, 255, 255)
    midgrey = gimpcolor.RGB(125, 125, 125)
	
    # set non UI shine plugin parameters
    shine_shadow_X, shine_shadow_Y, shine_shadow_size, shine_shadow_opacity, shine_keep_selection, shine_keep_layers = 0,0,0,0,0,0
	
    if use_bevel:
        bevel_mode = fn_set_bevel_mode(effect_mode_ptr)
    if bevel_mode_ptr == 0:
        # ie carved in bevel
        chisel_mode = 0
    else:
        # ie chiselled off bevel
        chisel_mode = 1
		
    if use_bevel and chisel_mode == 0:
        # if using carved in bevel swap azimuth direction
        effect_azimuth = effect_azimuth - 180
        if effect_azimuth < 0: effect_azimuth = 360 + effect_azimuth
		
	
    # set non UI chisel plugin parameters
    chisel_depth, chisel_location, chisel_keep_bump = 60, 0, 0

    loop_ptr = 1
    num_layers, layer_ids = pdb.gimp_image_get_layers(image)
    loop = num_layers - loop_ptr
    layer_names_list = image.layers
	
    visi_loop, other_layers, first_letter_layer, has_a_space = 0, 0, -1, False
    bumpmap_layer = "none"

    if use_texture:
        while visi_loop < num_layers:
            # check for layers to create bumpmap from
            visi_name = pdb.gimp_item_get_name(layer_names_list[visi_loop])
            visi_name = visi_name.decode('utf-8')
            if visi_name == "space.png" or visi_name == " .png": has_a_space = True
            if len(visi_name) == 1 or visi_name == "space.png" or len(visi_name) == 5 and ".png" in visi_name:
                if first_letter_layer < 0: first_letter_layer = visi_loop
            else:
                pdb.gimp_image_select_item(image, 2, layer_names_list[visi_loop])
                is_empty = pdb.gimp_selection_is_empty(image)
                if is_empty == False: 
                    other_layers += 1
                    if other_layers == 1: 
                        # Use the top-most non-character layer as the source for the bumpmap layer and make a copy of it
                        bumpmap_layer = layer_names_list[visi_loop].copy()
                        image.add_layer(bumpmap_layer, -1)
                        bumpmap_layer_name = "Bump"
                        pdb.gimp_image_set_active_layer(image, bumpmap_layer)
                        pdb.gimp_selection_none(image)
                        # desaturate the newly created bumpmap layer
                        pdb.gimp_drawable_desaturate( bumpmap_layer, 3)
                    		
				
            visi_loop += 1
        # Update the lists of layers
        loop_ptr = 1
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
        loop = num_layers - loop_ptr
        layer_names_list = image.layers
	
    if other_layers == 0: use_texture = False # if there are no layers to use for the bumpmap then cancel it
	
    first_char_layer_name = "none"
	
    if not (use_bevel == False and use_shine == False and use_emboss == False and use_texture == False):
        # turn off visibility of layers
        visi_loop, first_char_layer  = 0, -1
        while visi_loop < num_layers:
	        # get info about layer we are working on
            layer_name = pdb.gimp_item_get_name(layer_names_list[visi_loop])
            layer_name = layer_name.decode('utf-8')
            if len(layer_name) == 1 or len(layer_name) == 5 and ".png" in layer_name:
                if first_char_layer < 0: 
                    first_char_layer = visi_loop
                    first_char_layer_name = layer_name
            pdb.gimp_item_set_visible(layer_names_list[visi_loop],False)
            visi_loop += 1
	
    while loop >= 0:
	    # get info about layer we are working on
        layer_id = layer_ids[loop]
        layer_name = layer_names_list[loop]
	
        # cater for unicode characters above normal range
        display_layer_name =  pdb.gimp_item_get_name(layer_names_list[loop])
        display_layer_name = display_layer_name.decode('utf-8')	
	
        # check we have the layer id of the currently active layer
        pdb.gimp_image_set_active_layer(image, layer_name)
        layer_id = pdb.gimp_image_get_active_layer(image)

        # check to see if the current layer is a custom font character layer
        test_for_custom_layer = fn_test_for_custom_layer(image, layer_id, display_layer_name)
		
        # if the outline width has been set to zero but no chisel or shine has been set don't do it
        #if use_chisel == False and use_shine == False: test_for_custom_layer = False
		
        if test_for_custom_layer:

            # ensure we have the active layer and make it visible
            active_layer_id = pdb.gimp_image_get_active_layer(image)
            pdb.gimp_item_set_visible(active_layer_id, True)

            if use_bevel:
                soften = int(image.height * 0.00625)
                active_layer = image.active_layer 
                active_layer_id = proc_text_bevel(image, active_layer, bevel_width, bevel_depth, bevel_waterlevel, bevel_ambient, soften, black, white, midgrey, bevel_mode, effect_azimuth, effect_elevation, blur_radius, blur_delta)
				
            if use_emboss:
                active_layer = image.active_layer
                active_layer_id = proc_emboss(image, active_layer, black, white, emboss_width, effect_elevation, effect_azimuth, emboss_blur, effect_mode_ptr)
				
            if use_shine:
                active_layer = image.active_layer 
                active_layer_id = proc_gloss(image, active_layer, black, white, effect_azimuth, gloss_colour, trim_value)	
  
				
            if use_texture:
                active_layer = image.active_layer
                active_layer_id = proc_do_bump(image, other_layers, bumpmap_layer, bumpmap_layer_name, active_layer, black, white, bump_depth, bump_waterlevel, bump_ambient, effect_elevation, effect_azimuth, bevel_mode_ptr)		
				              
				
            if (num_layers - loop_ptr) > 0: pdb.gimp_item_set_visible(active_layer_id, False)
			
            num_layers, layer_ids = pdb.gimp_image_get_layers(image)
			
        loop_ptr += 1
        loop = num_layers - loop_ptr
        
    if use_texture: 
        image.remove_layer(bumpmap_layer)
        active_layer = image.active_layer 

    if first_char_layer_name <> "none":
        first_char_layer_id = pdb.gimp_image_get_layer_by_name(image, first_char_layer_name)
        pdb.gimp_item_set_visible(first_char_layer_id, True)
        pdb.gimp_image_set_active_layer(image,first_char_layer_id)
	
    # Set end undo point
    pdb.gimp_image_undo_group_end(image)

    # Restore the GIMP's settings
    gimp.context_pop()
		
#### End of do_stuff

######################################################################################################################################

def proc_do_bump(image, other_layers, bumpmap_layer_id, bumpmap_layer_name, active_layer, black, white, bumpmap_width, waterlevel, ambient, effect_elevation, effect_azimuth, bevel_mode_ptr):
    if other_layers > 0:
        work_layer = active_layer
        work_layer_name = work_layer.name
        width = work_layer.width
        height = work_layer.height
        bumpmap_width = bumpmap_width * 0.65
        if bumpmap_width < 1: bumpmap_width = 1
		
        if waterlevel > 0:
            waterlevel = waterlevel/100
            waterlevel = 1.0 - waterlevel
        else:
            waterlevel = 0.0
			
        if ambient > 0:
            ambient = ambient/100
            ambient = 1.0 - ambient
        else:
            ambient = 0.0        

        pdb.plug_in_bump_map(image, work_layer, bumpmap_layer_id, effect_azimuth, effect_elevation, bumpmap_width, 0, 0, waterlevel, ambient, True, False, 0)
		
        active_layer = image.active_layer 
        return active_layer
        
######################################################################################################################################
	
def proc_emboss(image, active_layer, black, white, effect_width, effect_elevation, effect_azimuth, emboss_blur, bevel_mode_ptr):
    if effect_width < 1: effect_width = 1
    grey = gimpcolor.RGB(80, 80, 80)
    work_layer = active_layer
    work_layer_name = work_layer.name
    width = work_layer.width
    height = work_layer.height
    colour = gimp.get_background()
    emboss_mode = fn_set_bevel_mode(bevel_mode_ptr)
	
    layer_blur = image.height * 0.015
    if layer_blur < 1: layer_blur = 1	
	
    pdb.gimp_context_set_foreground(black)
    pdb.gimp_context_set_background(white)
	
    pdb.gimp_image_select_item(image, 2, work_layer)
	
    new_layer = gimp.Layer(image, "Emboss Layer", width, height, RGBA_IMAGE, 100, 0)
    image.add_layer(new_layer, -1)
    new_layer.fill(1)
    pdb.gimp_drawable_edit_fill(new_layer, 0)

    pdb.gimp_image_set_active_layer(image, new_layer)
    pdb.gimp_item_set_name(work_layer,"Original layer")
	
    pdb.gimp_context_set_foreground(black)
    pdb.gimp_context_set_background(white)
	
    #pdb.plug_in_gauss_rle2(image, new_layer, layer_blur, layer_blur)
    pdb.plug_in_gauss_iir(image, new_layer, emboss_blur, True, True)
    pdb.plug_in_emboss(image, new_layer, effect_azimuth, effect_elevation, effect_width, 1)
    pdb.gimp_selection_invert(image)
    pdb.gimp_drawable_edit_clear(new_layer)
    pdb.gimp_selection_invert(image)

    pdb.plug_in_gauss_iir(image, new_layer, emboss_blur, True, True)
    #pdb.plug_in_gauss_rle2(image, new_layer, emboss_blur, emboss_blur)
    new_layer.mode = emboss_mode
    pdb.gimp_item_set_name(work_layer, work_layer_name)
    pdb.gimp_image_merge_down(image, new_layer, 0)
    new_layer = image.active_layer
		
    pdb.gimp_selection_none(image)
    active_layer = image.active_layer 
    return active_layer	
	
######################################################################################################################################

def proc_gloss(image, active_layer, black, white, effect_azimuth, gloss_colour, trim_value):
    work_layer = active_layer
    work_layer_name = work_layer.name
    width = work_layer.width
    height = work_layer.height
    elevation = 85
    emboss_width = 10
    colour = gimp.get_background()
	
    layer_blur = image.height * 0.015
    if layer_blur < 1: layer_blur = 1	
	
    pdb.gimp_context_set_foreground(black)
    pdb.gimp_context_set_background(white)
	
    pdb.gimp_image_select_item(image, 2, work_layer)
	
    new_layer = gimp.Layer(image, "Shine Layer", width, height, RGBA_IMAGE, 100, 0)
    image.add_layer(new_layer, -1)
    new_layer.fill(1)
    pdb.gimp_drawable_edit_fill(new_layer, 0)

    pdb.gimp_image_set_active_layer(image, new_layer)
    pdb.gimp_item_set_name(work_layer,"Original layer")
	
    # apply shine
    pdb.gimp_context_set_foreground(black)
    pdb.gimp_context_set_background(white)
	
    pdb.plug_in_gauss_rle2(image, new_layer, layer_blur, layer_blur)
    pdb.plug_in_emboss(image, new_layer, effect_azimuth, elevation, emboss_width, 1)
    pdb.gimp_selection_invert(image)
    pdb.gimp_drawable_edit_clear(new_layer)
    pdb.gimp_selection_invert(image)
    pdb.plug_in_colortoalpha(image ,new_layer ,(254, 254, 254))
    
    if gloss_colour:
        sel_is_empty = pdb.gimp_selection_is_empty(image) # better check that the selection still exists before shrinking
        if not sel_is_empty:
            pdb.gimp_context_set_background(colour) # get bg colour
            colour_layer = gimp.Layer(image, "Colour Layer", width, height, RGBA_IMAGE, 100, 35) # add a darken only mode layer
            pdb.gimp_selection_shrink(image, 1) # shrink selection boundary by 1 pixel
            image.add_layer(colour_layer, -1)
            colour_layer.fill(1) # fill the layer selection
            pdb.gimp_image_merge_down(image, colour_layer, 0)
            new_layer = image.active_layer # reset layer id
	
    pdb.plug_in_blur(image, new_layer)
    pdb.gimp_item_set_name(work_layer, work_layer_name)
    pdb.gimp_image_merge_down(image, new_layer, 0)
		
    pdb.gimp_selection_none(image)
    new_layer = image.active_layer
    proc_trim_away(image, new_layer, trim_value)# shrink selection boundary by set amount of pixels
    active_layer = image.active_layer 
    return active_layer

    #### End of proc_gloss
	
######################################################################################################################################

def proc_text_bevel(image, text_layer, bevel_width, bevel_depth, bevel_waterlevel, bevel_ambient, feather, black, white, midgrey, bevel_mode, bevel_azimuth, bevel_elevation, blur_radius, blur_delta):
    #### Create a bevel around text and return the new layer as bevel_layer
    #Rem Adjust values taken from dialog
    bevel_width = int(bevel_width * 2.55)
    if bevel_width < 2: bevel_width = 2
    blur_radius = blur_radius * 1
    blur_delta = blur_delta * 2.55
	
    if bevel_ambient > 0.0:
        ambient = bevel_ambient/100
    else:
        ambient = 0.0
    if bevel_waterlevel > 0.0:
        waterlevel = bevel_waterlevel/100
    else:
        waterlevel = 0.0
		
    #Rem Select text outline and save it
    pdb.gimp_image_select_item(image, 2, text_layer)
    saved_selection = pdb.gimp_selection_save(image)

    #### Rem Make a bumpmap layer ######
    bumpmap_layer = text_layer.copy()
    image.add_layer(bumpmap_layer, -1)
    bumpmap_layer.name = text_layer.name + " Bumpmap"
    pdb.gimp_image_raise_item_to_top(image, bumpmap_layer)

    pdb.gimp_selection_feather(image, feather)
    blurred_selection = pdb.gimp_selection_save(image)
    loop = bevel_width
    pdb.gimp_selection_shrink(image, loop)

    while (pdb.gimp_selection_is_empty(image) == True ):
        loop = loop - 1
        pdb.gimp_image_select_item(image, 2, blurred_selection)
        pdb.gimp_selection_shrink(image, loop)
        
    bevel_width = min(bevel_width, loop)
    pdb.gimp_image_select_item(image, 2, blurred_selection)

    gimp.set_foreground(0, 0, 0)
    bumpmap_layer.fill(0)
	

    loop = 1
    #Rem Temp dirty fill_value 
    fill_value = (255 / bevel_width)
    while (loop <= bevel_width):

        #Rem Using Temp fill_value until something better comes to mind
        pdb.gimp_image_select_item(image, 2, blurred_selection)
        pdb.gimp_selection_shrink(image, loop -1)
        greycolour = int(fill_value)
        gimp.set_foreground(greycolour, greycolour, greycolour)

        pdb.gimp_edit_fill(bumpmap_layer, 0)

        #Rem Temp fill_value
        fill_value = fill_value + (255 / bevel_width)
        loop = loop + 1

    gimp.set_foreground(255, 255, 255)
    pdb.gimp_image_select_item(image, 2, blurred_selection)
    pdb.gimp_selection_shrink(image, bevel_width)

    if (pdb.gimp_selection_is_empty(image) == FALSE):
        pdb.gimp_edit_fill(bumpmap_layer, 0)

    pdb.gimp_selection_none(image)

    #### Make Bevel from bumpmap ####
    bevel_layer = text_layer.copy()
    image.add_layer(bevel_layer, -1)
    bevel_layer.name = text_layer.name + " Bevel"
    pdb.gimp_image_raise_item_to_top(image, bevel_layer)

    gimp.set_foreground(125, 125, 125)
    bevel_layer.fill(0)
    bevel_azimuth = bevel_azimuth - 180
    if bevel_azimuth < 0: bevel_azimuth = 360 + (bevel_azimuth)
    pdb.plug_in_bump_map(image, bevel_layer, bumpmap_layer, bevel_azimuth, bevel_elevation, bevel_depth, 0, 0, waterlevel, ambient, True, False, 0)
    bevel_layer.mode = bevel_mode
    pdb.gimp_layer_set_opacity(bevel_layer, 80)
	
    #### Delete area outside the bevel ####
    pdb.gimp_image_select_item(image, 2, saved_selection)
    pdb.gimp_selection_invert(image)

    if (pdb.gimp_selection_is_empty(image) == False):
        pdb.gimp_edit_clear(bevel_layer)

    pdb.gimp_image_select_item(image, 2, saved_selection)
    pdb.gimp_selection_shrink(image, bevel_width)

    if (pdb.gimp_selection_is_empty(image) == False):
        pdb.gimp_edit_clear(bevel_layer)
	
    pdb.gimp_image_select_item(image, 2, saved_selection)
 
    pdb.gimp_selection_none(image)
    pdb.gimp_drawable_set_visible(bumpmap_layer, False)	
    image.remove_channel(saved_selection)
    image.remove_channel(blurred_selection)
    image.remove_layer(bumpmap_layer)
    pdb.gimp_image_merge_down(image, bevel_layer, 0)
    active_layer = image.active_layer 
    pdb.plug_in_sel_gauss(image, active_layer, blur_radius, blur_delta)
    active_layer = image.active_layer 
    return active_layer
	
#### End of proc_text_bevel

######################################################################################################################################
	
def proc_trim_away(img, layer, amount):    
    # select the contents of the layer
    pdb.gimp_image_select_item(img, 2, layer)
    # shrink the selection
    pdb.gimp_selection_shrink(img, amount)
    # invert selection
    pdb.gimp_selection_invert(img)
    # trim away pixels
    successful = pdb.gimp_edit_cut(layer)
    # deselect
    pdb.gimp_selection_none(img)
    
######################################################################################################################################

def fn_test_for_custom_layer(img,l_id,l_name):
    test = False
    if len(l_name) == 1:
        test = True
    elif len(l_name) == 5:
        if ".png" in l_name:
            test = True
    # select the contents of the layer
    pdb.gimp_image_select_item(img,2,l_id)
    # Test to see if the selection actually contains something
    sel_is_empty = pdb.gimp_selection_is_empty(img)
    # If there are no pixels to work with flag the layer as not viable to work on
    if sel_is_empty: test = False
    # Deselect
    pdb.gimp_selection_none(img)
    return test
	
######################################################################################################################################
	
def fn_set_bevel_mode(bevel_mode_ptr):
    # Use New Modes when creating a bevel
    bevel_mode = 23
    if bevel_mode_ptr == 0:
        bevel_mode = 23 #Overlay Mode
    elif bevel_mode_ptr == 1:
        bevel_mode = 30 #Multiply Mode
    elif bevel_mode_ptr == 2:
        bevel_mode = 35 #Darken Only Mode
    elif bevel_mode_ptr == 3:
        bevel_mode = 40 #HSV Value Mode
    elif bevel_mode_ptr == 4:
        bevel_mode = 41 #Divide Mode
    elif bevel_mode_ptr == 5:
        bevel_mode = 44 # Hardlight Mode
    elif bevel_mode_ptr == 6:
        bevel_mode = 45 # Softlight Mode
    if bevel_mode_ptr == 7:
        bevel_mode = 46 #Grain Extract Mode
    elif bevel_mode_ptr == 8:
        bevel_mode = 47 #Grain Merge Mode
    elif bevel_mode_ptr == 9:
        bevel_mode = 48 #Vivid Mode
    elif bevel_mode_ptr == 10:
        bevel_mode = 50 #Linear Light Mode
    elif bevel_mode_ptr == 11:
        bevel_mode = 51 #Hard Mix Mode
    elif bevel_mode_ptr == 12:
        bevel_mode = 56 #Luminance Mode
    return bevel_mode


######################################################################################################################################

register(
    "python_fu_Custom_Font_Effects_Only_v0-04",
    "Adds effects to custom fonts",
    "Provides options for adding batch effects to custom fonts",
    "Skinnyhouse",
    "Open source (BSD 3-clause license)",
    "2018",
    "<Image>/Filters/Custom Font Tools/Custom Font Effects Only v0.04...",
    "*",
    [
    (PF_TOGGLE, "pf_add_emboss", "Add Emboss: ", 0),
    (PF_SPINNER, "pf_emboss_width", "Emboss width: ", 30,(0,100,1)),
    (PF_SPINNER, "pf_emboss_blur", "Emboss blur: ", 5,(0,100,1)),
    (PF_TOGGLE, "pf_add_bump", "Add Bumpmapped Texture: ", 0),
    (PF_SPINNER, "pf_bump_depth", "Bumpmap depth (percentage): ", 30,(0,100,1)),
    (PF_SPINNER, "pf_bump_waterlevel", "Bumpmap waterlevel (percentage): ", 0,(0,100,1)),
    (PF_SPINNER, "pf_bump_ambient", "Bumpmap ambient light (percentage): ", 0,(0,100,1)),
    (PF_TOGGLE, "pf_add_shine", "Add Shine effect: ", 0),
    (PF_SPINNER, "pf_shine_trim", "Trim shine in pixels: ", 0,(0,10,1)),
    (PF_TOGGLE, "pf_shine_colour", "Use (BG) colour for Shine: ", 0),
    (PF_TOGGLE, "pf_add_bevel", "Add a bevel: ", 0),
    (PF_TOGGLE, "pf_bevel_out_or_bevel_in_mode", "Bevel out (Yes) or bevel in (No): ", 1),
    (PF_SPINNER, "pf_bevel_width", "Bevel width: ", 30,(0,100,1)),
    (PF_SPINNER, "pf_bevel_depth", "Bevel depth: ", 40,(0,100,1)),
    (PF_SPINNER, "pf_bevel_waterlevel", "Bevel waterlevel (percentage): ", 0,(0,100,1)),
    (PF_SPINNER, "pf_bevel_ambient", "Bevel ambient light (percentage): ", 0,(0,100,1)),
    (PF_SPINNER, "pf_blur_radius", "Bevel blur radius: ", 10,(0.0,100.0,0.1)),
    (PF_SPINNER, "pf_blur_delta", "Bevel blur delta amount: ", 8,(0,100,1)),
    (PF_SPINNER, "pf_effect_elevation", "Effect elevation: ", 45.0,(0.5,90,0.1)),
    (PF_SPINNER, "pf_shine_azimuth", "Effect azimuth: ", 315,(0,360,1)),
    (PF_OPTION, "pf_option", "Effect colour mode: ", 8, ("Overlay Mode", "Multiply Mode", "Darken Only Mode", "HSV Value Mode", "Divide Mode", "Dodge Mode", "Hardlight Mode", "Softlight Mode", "Grain Extract Mode", "Grain Merge Mode", "Vivid Light Mode", "Hard Mix Mode", "Luminance Mode")),
    ],
    [],
    do_stuff)

main()

	

