#!/usr/bin/env python
#
# -------------------------------------------------------------------------------------
#
# Copyright (c) 2018 KGH
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#
# This file is a basic Python plug-in for GIMP.
#
# It can be executed by selecting the menu option: 'Filters>>Custom Font Tool>>Custom Font Outlines v0.06'
#
# Change Log for Custom Font Outlines
# Version 0.01 Provision added to call Shine Filter v 0.01 or v 0.01b by Graechan and Chisel or Carve Filter version 1.3 by Rob Antonishen
# Version 0.02 Outline width set to zero will allow the filter to add shine and/or chisel to existing custom font layers without adding outlines when outline width set to zero
#              Distort will also work on all these outlines apart from the brushed outline when outline width set to zero and Distort selection spread value > 0
# Version 0.03 Trimmed pixels from letter faces to remove slight halo effect
# Version 0.04 Added gaussian blur after chisel effect to smooth artifacts using pdb.plug_in_sel_blur with options for blur radius and blur delta
# Version 0.05 Removed dependency on shine and chisel plug-ins and replaced calls with embedded code
#              Added option to colour the shiny gloss
# Version 0.01 Custom Font Outlines Only
#              Removed effects to reduce size of filter dialog
#              New filter menu item for Custom Font Tools
# Current Version of Custom Font Outlines 0.05
# Current Version of Custom Font Outlines Only 0.01

from gimpfu import *

#def do_stuff(image, layer, outline_gradient, outline_pattern, outline_width, stroke_with_brush, stroke_with_gradient, stroke_with_pattern, untrim, restrict_brush_size, merge_layers, outlines_only, keep_paths, 
#            distress_spread, distress_granularity, distress_smooth, azimuth, use_shine, gloss_colour, use_chisel, chisel_width, chisel_elevation, blur_radius, blur_delta, chisel_mode_ptr, bevel_mode_ptr, chisel_before_shine):
def do_stuff(image, layer, outline_gradient, outline_pattern, outline_width, stroke_with_brush, stroke_with_gradient, stroke_with_pattern, untrim, restrict_brush_size, merge_layers, outlines_only, keep_paths, 
            distress_spread, distress_granularity, distress_smooth):

    # Store the GIMP's settings so they can be restored
    gimp.context_push()
    # Set start undo point
    pdb.gimp_image_undo_group_start(image)
	
    use_clipboard_as_brush, do_layer_info, do_layer_names, do_path_names, do_sort = 0,0,0,0,0
    outline_colour = pdb.gimp_context_get_foreground()
	
    # Define some colours for proc chisel
    black = gimpcolor.RGB(0, 0, 0)
    white = gimpcolor.RGB(255, 255, 255)
    midgrey = gimpcolor.RGB(125, 125, 125)
	
    # set non UI shine plugin parameters
    shine_shadow_X, shine_shadow_Y, shine_shadow_size, shine_shadow_opacity, shine_keep_selection, shine_keep_layers = 0,0,0,0,0,0
		
    ###### Temp effects setting before removal
    use_chisel = False
    use_shine = False
	
    # set non UI chisel plugin parameters
    chisel_depth, chisel_location, chisel_keep_bump = 60, 0, 0

    loop_ptr = 1
    num_layers, layer_ids = pdb.gimp_image_get_layers(image)
    loop = num_layers - loop_ptr
    layer_names_list = image.layers
	
	
    first_char_layer_name = "none"
    # if the outline width has been set to zero but no chisel or shine has been set don't alter layers visibility
    if not (outline_width == 0 and use_chisel == False and use_shine == False):
        # turn off visibility of layers
        visi_loop, first_char_layer  = 0, -1
        while visi_loop < num_layers:
	        # get info about layer we are working on
            layer_name = pdb.gimp_item_get_name(layer_names_list[visi_loop])
            layer_name = layer_name.decode('utf-8')
            if len(layer_name) == 1 or len(layer_name) == 5 and ".png" in layer_name:
                if first_char_layer < 0: 
                    first_char_layer = visi_loop
                    first_char_layer_name = layer_name
            pdb.gimp_item_set_visible(layer_names_list[visi_loop],False)
            visi_loop += 1
	
    # set value of distress_threshold
    distress_threshold = 127
	
	
    # set stroke method to paint line  or with the currently selected brush
    if stroke_with_brush:
        stroke_method = 1
    else:
        stroke_method = 0
		
    # Set various stroke options
    pdb.gimp_context_set_line_width(outline_width)
    pdb.gimp_context_set_stroke_method(stroke_method)
    pdb.gimp_context_set_foreground(outline_colour)
    pdb.gimp_context_set_sample_transparent(True)
    current_brush = pdb.gimp_context_get_brush()
	
    if stroke_method == 1:
        if restrict_brush_size:
            if outline_width > 0: pdb.gimp_context_set_brush_size(outline_width*1.75)
        if current_brush == "Clipboard Image":
            if not use_clipboard_as_brush:
                number_of_brushes, brushes_list = pdb.gimp_brushes_get_list("")
                if number_of_brushes > 2:
                    pdb.gimp_context_set_brush(brushes_list[2])
		
	
    while loop >= 0:
	    # get info about layer we are working on
        layer_id = layer_ids[loop]
        layer_name = layer_names_list[loop]
	
        # cater for unicode characters above normal range
        display_layer_name =  pdb.gimp_item_get_name(layer_names_list[loop])
        display_layer_name = display_layer_name.decode('utf-8')	
	
        # check we have the layer id of the currently active layer
        pdb.gimp_image_set_active_layer(image, layer_name)
        layer_id = pdb.gimp_image_get_active_layer(image)

        # check to see if the current layer is a custom font character layer
        test_for_custom_layer = fn_test_for_custom_layer(image, layer_id, display_layer_name)
		
        # if the outline width has been set to zero but no chisel or shine has been set don't do it
        if outline_width == 0 and use_chisel == False and use_shine == False: test_for_custom_layer = False
		
        if outline_width == 0 and distress_spread > 0 and merge_layers:
            temp_outline_width = 20
            pdb.gimp_context_set_line_width(temp_outline_width)
        else:
            temp_outline_width = 0
		
        if test_for_custom_layer:

            # ensure we have the active layer and make it visible
            active_layer_id = pdb.gimp_image_get_active_layer(image)
            pdb.gimp_item_set_visible(active_layer_id, True)
			
            if outline_width > 0 or (outline_width == 0 and temp_outline_width == 20):
		
                # copy layer as outline layer
                outline_layer = pdb.gimp_layer_copy(layer_name, True)
		
                # insert new outline layer above the active layer
                pdb.gimp_image_insert_layer(image, outline_layer, None, -1)
		
                # name the new outline layer
                pdb.gimp_item_set_name(outline_layer, str(display_layer_name[0] + " outline") )
		
                # select the contents of the layer
                pdb.gimp_image_select_item(image, 2, outline_layer)
		
                # shrink the selection by the required amount
                if outline_width > 1 :pdb.gimp_selection_shrink(image, int(outline_width/2) - 1)
                if temp_outline_width == 20 :pdb.gimp_selection_shrink(image, int(temp_outline_width/2) - 1)
			
                # distort path if spread > 0
                if distress_spread > 0: pdb.script_fu_distress_selection(image, outline_layer, distress_threshold, distress_spread, distress_granularity, distress_smooth, True, True)
				
                # create path from current selection
                pdb.plug_in_sel2path(image, outline_layer)
		
                # get id of the newly created path
                active_vector_id = pdb.gimp_image_get_active_vectors(image)
		
                # give the path its name
                pdb.gimp_item_set_name(active_vector_id, str(display_layer_name[0] + " path") )
		
                # Deselect
                pdb.gimp_selection_none(image)
		
                # clear the contents of the layer
                pdb.gimp_drawable_edit_clear(outline_layer)
		
            if stroke_with_gradient:
                if outline_width > 0 or temp_outline_width == 20:
                    # create an outline using the current gradient
                    # Force stroke method to stroke with line tool
                    pdb.gimp_context_set_stroke_method(0)
				
                    # stroke the path (using current width/colour settings)
                    pdb.gimp_drawable_edit_stroke_item(outline_layer, active_vector_id)
				
                    # Make a selection from the outline to create a border selection
                    pdb.gimp_image_select_item(image, 2, outline_layer)

                    # fill the selection with the gradient
                    pdb.gimp_context_set_gradient(outline_gradient)
                    pdb.gimp_drawable_edit_gradient_fill(outline_layer, 0, 0, False, 1, 0, True, (outline_layer.width)/2, (outline_layer.height), (outline_layer.width)/2, 0)
				
                    # Deselect
                    pdb.gimp_selection_none(image)
				
            elif stroke_with_pattern:
                if outline_width > 0 or temp_outline_width == 20:
                    # create an outline using the current pattern			
                    # Force stroke method to stroke with line tool
                    pdb.gimp_context_set_stroke_method(0)
				
                    # stroke the path (using current width/colour settings)
                    pdb.gimp_drawable_edit_stroke_item(outline_layer, active_vector_id)
				
                    # Make a selection from the outline to create a border selection
                    pdb.gimp_image_select_item(image, 2, outline_layer)

                    # fill the selection with the gradient
                    pdb.gimp_context_set_pattern(outline_pattern)
                    pdb.gimp_drawable_edit_fill(outline_layer, 4)
				
                    # Deselect
                    pdb.gimp_selection_none(image)
                
            else:
                if outline_width > 0 or temp_outline_width == 20:
                    # create an outline using the currently selected brush or as a simple line
                    # set the gradient blend colour space ?
                    pdb.gimp_context_set_gradient_blend_color_space(0)
			
                    # stroke the path (using current width/colour settings)
                    pdb.gimp_drawable_edit_stroke_item(outline_layer, active_vector_id)
			
            if outline_width > 0 or temp_outline_width == 20:
                # turn on the visibility of the new outline layer for all types of outline
                pdb.gimp_item_set_visible(outline_layer, True)
                #dummy = hot
			
                if not keep_paths: pdb.gimp_image_remove_vectors(image,active_vector_id)
			
            if merge_layers:
                if not stroke_with_brush:
                    if (not untrim and outline_width > 0 or not untrim and temp_outline_width == 20):
                        pdb.gimp_image_select_contiguous_color(image, 2, outline_layer, 0, 0)
                        pdb.gimp_selection_grow(image, 1)
                        successful = pdb.gimp_edit_cut(active_layer_id)
                        pdb.gimp_selection_none(image)
                if outline_width > 0:
                    pdb.gimp_image_merge_down(image, outline_layer, 0)
                if temp_outline_width == 20:
                    # clear the contents of the layer
                    pdb.gimp_drawable_edit_clear(outline_layer)
                    pdb.gimp_image_merge_down(image, outline_layer, 0)
                active_layer_id = pdb.gimp_image_get_active_layer(image)			
				
                if (num_layers - loop_ptr) > 0: pdb.gimp_item_set_visible(active_layer_id, False)
            elif outlines_only:
                if outline_width > 0:
                    pdb.gimp_drawable_edit_clear(active_layer_id)
                    pdb.gimp_image_merge_down(image, outline_layer, 0)
                    active_layer_id = pdb.gimp_image_get_active_layer(image)
                    if (num_layers - loop_ptr) > 0: pdb.gimp_item_set_visible(active_layer_id, False)
            else:
                brush_size = pdb.gimp_context_get_brush_size()
                if (num_layers - loop_ptr) > 0:
                    #if not stroke_with_brush:
                    if not untrim and outline_width > 0 or not untrim and temp_outline_width == 20:
                        pdb.gimp_image_select_contiguous_color(image, 2, outline_layer, 0, 0)
                        if stroke_with_brush: 
                            pdb.gimp_selection_grow(image, brush_size/4) 
                        else: 
                            pdb.gimp_selection_grow(image, outline_width/4)
                        successful = pdb.gimp_edit_cut(active_layer_id)
                        pdb.gimp_selection_none(image)
                    if outline_width > 0 or temp_outline_width == 20: pdb.gimp_item_set_visible(active_layer_id, False)	
                    if outline_width > 0 or temp_outline_width == 20: pdb.gimp_item_set_visible(outline_layer, False)	
                elif (num_layers - loop_ptr) == 0:
                    #if not stroke_with_brush:
                    if not untrim and outline_width > 0 or not untrim and temp_outline_width == 20:
                        pdb.gimp_image_select_contiguous_color(image, 2, outline_layer, 0, 0)
                        if stroke_with_brush: 
                            pdb.gimp_selection_grow(image, brush_size/4) 
                        else: 
                            pdb.gimp_selection_grow(image, outline_width/4)
                        successful = pdb.gimp_edit_cut(active_layer_id)
                        pdb.gimp_selection_none(image)
                loop_ptr += 1
			
            num_layers, layer_ids = pdb.gimp_image_get_layers(image)
			
        loop_ptr += 1
        loop = num_layers - loop_ptr
        

    if first_char_layer_name <> "none":
        if merge_layers or outlines_only:
            first_char_layer_id = pdb.gimp_image_get_layer_by_name(image, first_char_layer_name)
            pdb.gimp_item_set_visible(first_char_layer_id, True)
            pdb.gimp_image_set_active_layer(image,first_char_layer_id)
        else:
            first_char_layer_id = pdb.gimp_image_get_layer_by_name(image, first_char_layer_name)
            pdb.gimp_item_set_visible(first_char_layer_id, True)
            if outline_width > 0:first_char_layer_name = str(first_char_layer_name[0]) + " outline"
            if outline_width > 0:first_char_layer_id = pdb.gimp_image_get_layer_by_name(image, first_char_layer_name)
            if outline_width > 0:pdb.gimp_item_set_visible(first_char_layer_id, True)
            pdb.gimp_image_set_active_layer(image,first_char_layer_id)
	
    # Set end undo point
    pdb.gimp_image_undo_group_end(image)

    # Restore the GIMP's settings
    gimp.context_pop()
		
#### End of do_stuff

######################################################################################################################################
	
def proc_trim_away(img, layer, amount):    
    # select the contents of the layer
    pdb.gimp_image_select_item(img, 2, layer)
    # shrink the selection
    pdb.gimp_selection_shrink(img, amount)
    # invert selection
    pdb.gimp_selection_invert(img)
    # trim away pixels
    successful = pdb.gimp_edit_cut(layer)
    # deselect
    pdb.gimp_selection_none(img)
    
######################################################################################################################################

def fn_test_for_custom_layer(img,l_id,l_name):
    test = False
    if len(l_name) == 1:
        test = True
    elif len(l_name) == 5:
        if ".png" in l_name:
            test = True
    # select the contents of the layer
    pdb.gimp_image_select_item(img,2,l_id)
    # Test to see if the selection actually contains something
    sel_is_empty = pdb.gimp_selection_is_empty(img)
    # If there are no pixels to work with flag the layer as not viable to work on
    if sel_is_empty: test = False
    # Deselect
    pdb.gimp_selection_none(img)
    return test
	
######################################################################################################################################
	
def proc_gloss(image, active_layer, black, white, azimuth, gloss_colour):
    work_layer = active_layer
    work_layer_name = work_layer.name
    width = work_layer.width
    height = work_layer.height
    elevation = 85
    emboss_width = 10
    colour = gimp.get_background()
	
    layer_blur = image.height * 0.015
    if layer_blur < 1: layer_blur = 1	
	
    pdb.gimp_context_set_foreground(black)
    pdb.gimp_context_set_background(white)
	
    pdb.gimp_image_select_item(image, 2, work_layer)
	
    new_layer = gimp.Layer(image, "Shine Layer", width, height, RGBA_IMAGE, 100, 0)
    image.add_layer(new_layer, -1)
    new_layer.fill(1)
    pdb.gimp_drawable_edit_fill(new_layer, 0)

    pdb.gimp_image_set_active_layer(image, new_layer)
    pdb.gimp_item_set_name(work_layer,"Original layer")
	
    # apply shine
    pdb.gimp_context_set_foreground(black)
    pdb.gimp_context_set_background(white)
	
    pdb.plug_in_gauss_rle2(image, new_layer, layer_blur, layer_blur)
    pdb.plug_in_emboss(image, new_layer, azimuth, elevation, emboss_width, 1)
    pdb.gimp_selection_invert(image)
    pdb.gimp_drawable_edit_clear(new_layer)
    pdb.gimp_selection_invert(image)
    pdb.plug_in_colortoalpha(image ,new_layer ,(254, 254, 254))

    
    if gloss_colour:
        sel_is_empty = pdb.gimp_selection_is_empty(image) # better check that the selection still exists before shrinking
        if not sel_is_empty:
            pdb.gimp_context_set_background(colour) # get bg colour
            colour_layer = gimp.Layer(image, "Colour Layer", width, height, RGBA_IMAGE, 100, 35) # add a darken only mode layer
            pdb.gimp_selection_shrink(image, 1) # shrink selection boundary by 1 pixel
            image.add_layer(colour_layer, -1)
            colour_layer.fill(1) # fill the layer selection
            pdb.gimp_image_merge_down(image, colour_layer, 0)
            new_layer = image.active_layer # reset layer id
	
    pdb.plug_in_blur(image, new_layer)
    pdb.gimp_item_set_name(work_layer, work_layer_name)
    pdb.gimp_image_merge_down(image, new_layer, 0)
		
    pdb.gimp_selection_none(image)
    active_layer = image.active_layer 
    return active_layer

    #### End of proc_gloss
	
######################################################################################################################################

def proc_text_bevel(image, text_layer, bevel_width, bevel_depth, feather, black, white, midgrey, bevel_mode, bevel_azimuth, bevel_elevation, blur_radius, blur_delta):
    #### Create a bevel around text and return the new layer as bevel_layer
    #Rem Select text outline and save it
    pdb.gimp_image_select_item(image, 2, text_layer)
    saved_selection = pdb.gimp_selection_save(image)

    #### Rem Make a bumpmap layer ######
    bumpmap_layer = text_layer.copy()
    image.add_layer(bumpmap_layer, -1)
    bumpmap_layer.name = text_layer.name + " Bumpmap"
    pdb.gimp_image_raise_item_to_top(image, bumpmap_layer)

    pdb.gimp_selection_feather(image, feather)
    blurred_selection = pdb.gimp_selection_save(image)
    loop = bevel_width
    pdb.gimp_selection_shrink(image, loop)

    while (pdb.gimp_selection_is_empty(image) == True ):
        loop = loop - 1
        pdb.gimp_image_select_item(image, 2, blurred_selection)
        pdb.gimp_selection_shrink(image, loop)
        
    bevel_width = min(bevel_width, loop)
    pdb.gimp_image_select_item(image, 2, blurred_selection)

    gimp.set_foreground(0, 0, 0)
    bumpmap_layer.fill(0)
	

    loop = 1
    #Rem Temp dirty fill_value 
    fill_value = (255 / bevel_width)
    while (loop <= bevel_width):

        #Rem Using Temp fill_value until something better comes to mind
        pdb.gimp_image_select_item(image, 2, blurred_selection)
        pdb.gimp_selection_shrink(image, loop -1)
        greycolour = int(fill_value)
        gimp.set_foreground(greycolour, greycolour, greycolour)

        pdb.gimp_edit_fill(bumpmap_layer, 0)

        #Rem Temp fill_value
        fill_value = fill_value + (255 / bevel_width)
        loop = loop + 1

    gimp.set_foreground(255, 255, 255)
    pdb.gimp_image_select_item(image, 2, blurred_selection)
    pdb.gimp_selection_shrink(image, bevel_width)

    if (pdb.gimp_selection_is_empty(image) == FALSE):
        pdb.gimp_edit_fill(bumpmap_layer, 0)

    pdb.gimp_selection_none(image)

    #### Make Bevel from bumpmap ####
    bevel_layer = text_layer.copy()
    image.add_layer(bevel_layer, -1)
    bevel_layer.name = text_layer.name + " Bevel"
    pdb.gimp_image_raise_item_to_top(image, bevel_layer)

    gimp.set_foreground(125, 125, 125)
    bevel_layer.fill(0)
    bevel_azimuth = bevel_azimuth - 180
    if bevel_azimuth < 0: bevel_azimuth = 360 + (bevel_azimuth)
    pdb.plug_in_bump_map(image, bevel_layer, bumpmap_layer, bevel_azimuth, bevel_elevation, bevel_depth, 0, 0, 0, 0, True, False, 0)
    bevel_layer.mode = bevel_mode
    pdb.gimp_layer_set_opacity(bevel_layer, 80)
	
    #### Delete area outside the bevel ####
    pdb.gimp_image_select_item(image, 2, saved_selection)
    pdb.gimp_selection_invert(image)

    if (pdb.gimp_selection_is_empty(image) == False):
        pdb.gimp_edit_clear(bevel_layer)

    pdb.gimp_image_select_item(image, 2, saved_selection)
    pdb.gimp_selection_shrink(image, bevel_width)

    if (pdb.gimp_selection_is_empty(image) == False):
        pdb.gimp_edit_clear(bevel_layer)
	
    pdb.gimp_image_select_item(image, 2, saved_selection)
 
    pdb.gimp_selection_none(image)
    pdb.gimp_drawable_set_visible(bumpmap_layer, False)	
    image.remove_channel(saved_selection)
    image.remove_channel(blurred_selection)
    image.remove_layer(bumpmap_layer)
    pdb.gimp_image_merge_down(image, bevel_layer, 0)
    active_layer = image.active_layer 
    pdb.plug_in_sel_gauss(image, active_layer, blur_radius, blur_delta)
    active_layer = image.active_layer 
    return active_layer
	
#### End of proc_text_bevel

def fn_set_bevel_mode(bevel_mode_ptr):
    # Use New Modes when creating a bevel
    bevel_mode = 23
    if bevel_mode_ptr == 0:
        bevel_mode = 23 #Overlay Mode
    elif bevel_mode_ptr == 1:
        bevel_mode = 30 #Multiply Mode
    elif bevel_mode_ptr == 2:
        bevel_mode = 35 #Darken Only Mode
    elif bevel_mode_ptr == 3:
        bevel_mode = 40 #HSV Value Mode
    elif bevel_mode_ptr == 4:
        bevel_mode = 41 #Divide Mode
    elif bevel_mode_ptr == 5:
        bevel_mode = 44 # Hardlight Mode
    elif bevel_mode_ptr == 6:
        bevel_mode = 45 # Softlight Mode
    if bevel_mode_ptr == 7:
        bevel_mode = 46 #Grain Extract Mode
    elif bevel_mode_ptr == 8:
        bevel_mode = 47 #Grain Merge Mode
    elif bevel_mode_ptr == 9:
        bevel_mode = 48 #Vivid Mode
    elif bevel_mode_ptr == 10:
        bevel_mode = 50 #Linear Light Mode
    elif bevel_mode_ptr == 11:
        bevel_mode = 51 #Hard Mix Mode
    elif bevel_mode_ptr == 12:
        bevel_mode = 56 #Luminance Mode
    return bevel_mode


######################################################################################################################################

register(
    "python_fu_Custom_Font_Outlines_Only_v0-01",
    "Adds outlines to custom fonts",
    "Provides options for adding outlines to custom fonts",
    "Skinnyhouse",
    "Open source (BSD 3-clause license)",
    "2018",
    "<Image>/Filters/Custom Font Tools/Custom Font Outlines Only v0.01...",
    "*",
    [
    (PF_GRADIENT, "pf_current_gradient", "Gradient for outline:", 0),
    (PF_PATTERN, "pf_current_pattern", "Pattern for outline:", 0),
    (PF_SPINNER, "pf_outline_width", "Outline width in pixels: ",  10,(0,2000,1)),
    (PF_TOGGLE, "pf_stroke_method", "Stroke outline with \ncurrent brush: ", 0),
    (PF_TOGGLE, "pf_stroke_gradient", "Stroke outline with \ngradient: ", 0),
    (PF_TOGGLE, "pf_stroke_pattern", "Stroke outline with \npattern: ", 0),
    (PF_TOGGLE, "pf_untrim", "Un-trim letter layer: ", 0),
    (PF_TOGGLE, "pf_restrict_brush", "Constrain brush size: ", 0),
    (PF_TOGGLE, "pf_merge_layers", "Merge outline with \nits letter layer: ", 0),
    (PF_TOGGLE, "pf_outline_only", "Outlines only layers: ", 0),
    (PF_TOGGLE, "pf_keep_paths", "Retain letter paths: ", 0),
    (PF_SPINNER, "pf_distress_spread", "Distort selection spread value: ", 0,(0,1000,1)),
    (PF_SPINNER, "pf_distress_granularity", "Distort selection granularity value: ", 4,(1,25,1)),
    (PF_SPINNER, "pf_distress_smooth", "Distort selection smooth value: ", 2,(1,150,1)),
    ],
    [],
    do_stuff)

main()

	

